@@ -1,4 +1,6 @@
-# Copyright (C) 2003-2005 Nominum, Inc.
+# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license
+
+# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -19,7 +21,9 @@ import dns.exception
 import dns.rdata
 import dns.name
 
+
 class SOA(dns.rdata.Rdata):
+
     """SOA record
 
     @ivar mname: the SOA MNAME (master name) field
@@ -41,7 +45,7 @@ class SOA(dns.rdata.Rdata):
 
     __slots__ = ['mname', 'rname', 'serial', 'refresh', 'retry', 'expire',
                  'minimum']
-    
+
     def __init__(self, rdclass, rdtype, mname, rname, serial, refresh, retry,
                  expire, minimum):
         super(SOA, self).__init__(rdclass, rdtype)
@@ -58,9 +62,10 @@ class SOA(dns.rdata.Rdata):
         rname = self.rname.choose_relativity(origin, relativize)
         return '%s %s %d %d %d %d %d' % (
             mname, rname, self.serial, self.refresh, self.retry,
-            self.expire, self.minimum )
-        
-    def from_text(cls, rdclass, rdtype, tok, origin = None, relativize = True):
+            self.expire, self.minimum)
+
+    @classmethod
+    def from_text(cls, rdclass, rdtype, tok, origin=None, relativize=True):
         mname = tok.get_name()
         rname = tok.get_name()
         mname = mname.choose_relativity(origin, relativize)
@@ -72,18 +77,23 @@ class SOA(dns.rdata.Rdata):
         minimum = tok.get_ttl()
         tok.get_eol()
         return cls(rdclass, rdtype, mname, rname, serial, refresh, retry,
-                   expire, minimum )
-    
-    from_text = classmethod(from_text)
+                   expire, minimum)
 
-    def to_wire(self, file, compress = None, origin = None):
+    def to_wire(self, file, compress=None, origin=None):
         self.mname.to_wire(file, compress, origin)
         self.rname.to_wire(file, compress, origin)
         five_ints = struct.pack('!IIIII', self.serial, self.refresh,
                                 self.retry, self.expire, self.minimum)
         file.write(five_ints)
-        
-    def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin = None):
+
+    def to_digestable(self, origin=None):
+        return self.mname.to_digestable(origin) + \
+            self.rname.to_digestable(origin) + \
+            struct.pack('!IIIII', self.serial, self.refresh,
+                        self.retry, self.expire, self.minimum)
+
+    @classmethod
+    def from_wire(cls, rdclass, rdtype, wire, current, rdlen, origin=None):
         (mname, cused) = dns.name.from_wire(wire[: current + rdlen], current)
         current += cused
         rdlen -= cused
@@ -93,29 +103,14 @@ class SOA(dns.rdata.Rdata):
         if rdlen != 20:
             raise dns.exception.FormError
         five_ints = struct.unpack('!IIIII',
-                                  wire[current : current + rdlen])
-        if not origin is None:
+                                  wire[current: current + rdlen])
+        if origin is not None:
             mname = mname.relativize(origin)
             rname = rname.relativize(origin)
         return cls(rdclass, rdtype, mname, rname,
                    five_ints[0], five_ints[1], five_ints[2], five_ints[3],
                    five_ints[4])
 
-    from_wire = classmethod(from_wire)
-
-    def choose_relativity(self, origin = None, relativize = True):
+    def choose_relativity(self, origin=None, relativize=True):
         self.mname = self.mname.choose_relativity(origin, relativize)
         self.rname = self.rname.choose_relativity(origin, relativize)
-
-    def _cmp(self, other):
-        v = cmp(self.mname, other.mname)
-        if v == 0:
-            v = cmp(self.rname, other.rname)
-            if v == 0:
-                self_ints = struct.pack('!IIIII', self.serial, self.refresh,
-                                        self.retry, self.expire, self.minimum)
-                other_ints = struct.pack('!IIIII', other.serial, other.refresh,
-                                         other.retry, other.expire,
-                                         other.minimum)
-                v = cmp(self_ints, other_ints)
-        return v
