@@ -1,7 +1,9 @@
 #!/usr/local/bin/python2.7
 
+from __future__ import print_function
+
 import dns.resolver
 
 answers = dns.resolver.query('nominum.com', 'MX')
 for rdata in answers:
-    print 'Host', rdata.exchange, 'has preference', rdata.preference
+    print('Host', rdata.exchange, 'has preference', rdata.preference)
