@@ -1,6 +1,8 @@
 #!/usr/local/bin/python2.7
 
+from __future__ import print_function
+
 import dns.reversename
 n = dns.reversename.from_address("127.0.0.1")
-print n
-print dns.reversename.to_address(n)
+print(n)
+print(dns.reversename.to_address(n))
